;***********************************************************************************;
;***********************************************************************************;
;
; Wedge to load and save Scott Adams adventure games to disk.
;
; Copyright Simon Rowe <srowe@mose.org.uk>.
;
;***********************************************************************************;
;***********************************************************************************;

DDEV	= 8 			; disk device number

;***********************************************************************************;
;
; KERNAL zero page

STATUS	= $90			; I/O status byte
FNLEN	= $B7			; file name length
SA	= $B9			; secondary address
FA	= $BA			; current device number
FNADR	= $BB			; file name pointer

ILOAD	= $0330			; KERNAL vector - load
ISAVE	= $0332			; KERNAL vector - save

TBUFFR	= $033C			; to $03FB - cassette buffer
NAMSTO	= TBUFFR+$AF		; file name buffer

	; Adventure entrypoints
ADVST	= $7F50

	; KERNAL entrypoints

INITSK	= $E518			; initialize hardware
FLOAD2	= $F549			; KERNAL load routine
FSAVE2	= $F685			; KERNAL save routine
FRESTOR	= $FD52			; restore default I/O vectors
INITMEM	= $FD8D			; initialize and test RAM
INITVIA	= $FDF9			; initialize I/O registers
SETMSG	= $FF90			; control KERNAL messages
CHRIN	= $FFCF			; input character from channel (via vector)
CHROUT	= $FFD2			; output character to channel (via vector)
LOAD	= $FFD5			; load RAM from a device (via vector)
STOP	= $FFE1			; scan stop key (via vector)


;***********************************************************************************;
;***********************************************************************************;

#ifdef ROM
*	=	$A000

	.WORD	A8COLD
	.WORD	$FEC7		; normal warm start
	.BYT	"A0",'C'+$80,'B'+$80,'M'+$80
#else
*	=	TBUFFR-2
	.WORD	TBUFFR
#endif

;***********************************************************************************;
;
; wedge cold start entry point

A8COLD
#ifdef ROM
	JSR	INITMEM		; initialize and test RAM
	JSR	FRESTOR		; restore default I/O vectors
#endif
	LDA	#<A8LOAD	; get LOAD routine address low byte
	STA	ILOAD		; set LOAD vector low byte
	LDA	#>A8LOAD	; get LOAD routine address high byte
	STA	ILOAD+1		; set LOAD vector high byte
	LDA	#<A8SAVE	; get SAVE routine address low byte
	STA	ISAVE		; set SAVE vector low byte
	LDA	#>A8SAVE	; get SAVE routine address high byte
	STA	ISAVE+1		; set SAVE vector high byte
	JSR	INITVIA		; initialize I/O registers
	JSR	INITSK		; initialize hardware
	CLI			; enable interrupts
	LDA	#$80		; display control messages
	JSR	SETMSG		; control KERNAL messages
	LDA	#$00		; clear .A
	STA	STATUS		; clear I/O status byte
	JMP	ADVST		; start Adventure


;***********************************************************************************;
;
; wedge LOAD routine

A8LOAD
	JSR	GETNAM		; prompt for file name
	LDA	#00		; load operation
	INC	SA		; load to save address
	JSR	FLOAD2		; call normal KERNAL routine
	LDA	STATUS		; get I/O status byte
	AND	#$BF		; mask end of file bit
	STA	STATUS		; set I/O status byte
	RTS


;***********************************************************************************;
;
; wedge SAVE routine

A8SAVE
	JSR	GETNAM		; prompt for file name
	JMP	FSAVE2		; call normal KERNAL routine and return


;***********************************************************************************;
;
; prompt for file name, set zero page appropriately

GETNAM
	LDY	#$00		; clear .Y
BLOOP
	LDA	PROMPT,Y	; get prompt character
	BEQ	RDKEY		; end of string
	JSR	CHROUT
	INY
	BNE	BLOOP		; branch always

RDKEY	LDX	#0		; clear .X
PLOOP
	JSR	CHRIN		; input character from channel
	CMP	#13		; compare with [CR]
	BEQ	SETZP		; if [CR] set up zero page
	STA	NAMSTO,X	; save string to tape buffer
	INX			; increment index
	BNE	PLOOP		; repeat for next character

SETZP	STX	FNLEN		; set file name length
	LDA	#<NAMSTO	; get tape buffer start pointer low byte
	STA	FNADR		; set file name pointer low byte
	LDA	#>NAMSTO	; get tape buffer start pointer high byte
	STA	FNADR+1		; set file name pointer high byte
	LDA	#DDEV		; get disk device number
	STA	FA		; set device number
	RTS
	
PROMPT
	.BYTE	"eNTER FILE NAME:",$0D,$00
